-- Copyright (c) 2023-2024 Chris Pressey, Cat's Eye Technologies
-- This file is distributed under a BSD license.  See LICENSES directory:
-- SPDX-License-Identifier: LicenseRef-BSD-2-Clause-X-Fountain

module Language.Fountain.Store (
    Store, empty, fetch, insert, update, adjust, updateStore, trace
) where

--
-- Variable store.  This is not as general-purpose as it might sound.  It
-- not only contains values of variables, it can record a tracing log.
--

import qualified Data.Map as Map

import Language.Fountain.Constraint


data Store a = Store {
    store :: Map.Map Variable a,
    events :: [String]
} deriving (Show, Ord, Eq)

empty = Store{ store=Map.empty, events=[] }
fetch k s = Map.lookup k (store s)
insert k v s = s{ store=Map.insert k v $ store s }
update f k s = s{ store=Map.update f k $ store s }
adjust f k s = s{ store=Map.adjust f k $ store s }

updateStore :: [Variable] -> [Variable] -> Store a -> Store a -> Store a
updateStore [] [] _sourceStore destStore = destStore
updateStore [] _ _ _ = error "Variable lists must be same length"
updateStore _ [] _ _ = error "Variable lists must be same length"
updateStore (sourceKey:sourceKeys) (destKey:destKeys) sourceStore destStore =
    -- Populate destKey in the new store with the value at sourceKey in the sourceStore
    let
        destStore' = case fetch sourceKey sourceStore of
            Just val -> insert destKey val destStore
            Nothing -> destStore
    in
        updateStore sourceKeys destKeys sourceStore destStore'

trace str s = s{ events=(str:(events s)) }
